/**
 * Composite link plugin.
 */
new function () {
	
	var URL_LINK = "${tiny}/plugins/compositelink/link.aspx";
	
	tinymce.create ( "tinymce.plugins.CompositeLinkPlugin", {
		
		getInfo : function() {
			return {
				longname : "Composite Link",
				author : "Composite A/S",
				authorurl : "http://www.composite.net",
				infourl : null,
				version : tinymce.majorVersion + "." + tinymce.minorVersion
			};
		},
		
		/**
		 * @param {tinymce.Editor} ed
		 * @param {string} url
		 */
		init : function ( ed, url ) {

			this.editor = ed;

			ed.addShortcut('ctrl+k', '', 'compositeInsertLink');  
		},
		
		/**
		 * @param {HTMLAnchorElement} a
		 * @param {tinymce.Editor} editor
		 * @param {DataBindingResultMap} result
		 */
		_insertLink : function ( anchor, editor, result ) {
			
			//editor.getDoc().execCommand ( "unlink", false, null );

			editor.execCommand('mceInsertLink', false, {
				href: result.get("href"),
				rel: result.get("rel"),
				title: result.get("title"),
				tinymcetargetalias: result.get("blank") ? "_blank" : "",
				id: result.get( "id" ),
				"class": result.get("classname")
			});
		},
		
		
		/**
		 * @param {string} cmd
		 * @param {boolean} ui
		 * @param {string} value
		 */
		execCommand : function ( cmd, ui, value ) {
			
			var result = false;
			var self = this;
			var editor = this.editor;
			var editorBinding = editor.theme.editorBinding;
			
			if ( cmd == "unlink" ) {
				
				setTimeout ( function () {
					editorBinding.checkForDirty ();
					editorBinding.blurEditor ();
				}, 50 );
				// result is still false - relay to higher authority
				
			} else if ( cmd == "compositeInsertLink" ) {
				
				var elm = editor.selection.getNode ();
				var anchor = editor.dom.getParent ( elm, "a" );
				
				if ( value == null ) {
					value = anchor != null ? "update" : "insert";
				}
				 
				editorBinding.enableDialogMode ();
				
				var dialogHandler = {
					handleDialogResponse : function ( response, result ) {
					
						editorBinding.disableDialogMode ();
						
						if ( response == Dialog.RESPONSE_ACCEPT ) {		
							self._insertLink ( anchor, editor, result );
							editorBinding.getButtonForCommand ( "compositeInsertLink" ).disable ();
							editorBinding.getButtonForCommand ( "unlink" ).disable ();
							editorBinding.checkForDirty ();
							editorBinding.blurEditor ();
						}
					}
				};
				
				Dialog.invokeModal ( 
					URL_LINK,
					dialogHandler, 
					{
						tinyAction 		: value,
						tinyWindow 		: window,
						tinyElement 	: anchor,
						tinyEngine 		: tinymce.EditorManager,
						tinyInstance 	: editor,
						tinyTheme 		: editor.theme,
						editorBinding 	: editor.theme.editorBinding
					}
				);
				
				result = true;
			} 
			return result;
		}
	});

	// Register plugin
	tinymce.PluginManager.add("compositelink", tinymce.plugins.CompositeLinkPlugin);
};